@@ -1,6 +1,6 @@
 /* HP Scanjet 3900 series - Structures and global variables
 
-   Copyright (C) 2005-2006 Jonathan Bravo Lopez <jkdsoft@gmail.com>
+   Copyright (C) 2005-2007 Jonathan Bravo Lopez <jkdsoft@gmail.com>
 
    This file is part of the SANE package.
 
@@ -40,24 +40,87 @@
    If you do not wish that, delete this exception notice.
 */
 
-#define TOSHIBA     0x01
-#define SONY        0x00
+/* debugging level messages */
+#define DBG_ERR     0x00
+#define DBG_VRB     0x01
+#define DBG_FNC     0x02
+#define DBG_CTL     0x03
+#define DBG_BLK     0x04
 
+/* devices */
+#define HP3970      0x00	/* rts8822l-01H  HP Scanjet 3970 */
+#define HP4070      0x01	/* rts8822l-01H  HP Scanjet 4070 */
+#define HP4370      0x02	/* rts8822l-02A  HP Scanjet 4370 */
+#define UA4900      0x03	/* rts8822l-01H  UMAX Astra 4900 */
+#define HP3800      0x04	/* rts8822bl-03A HP Scanjet 3800 */
+
+/* chipset models */
+#define RTS8822L_01H   0x00
+#define RTS8822L_02A   0x01
+#define RTS8822BL_03A  0x02
+
+/* acceleration types */
+#define ACC_CURVE      0x00
+#define DEC_CURVE      0x01
+
+/* curve types */
+#define CRV_NORMALSCAN 0x00
+#define CRV_PARKHOME   0x01
+#define CRV_SMEARING   0x02
+#define CRV_BUFFERFULL 0x03
+
+/* Sample rates */
+#define PIXEL_RATE  0x00
+#define LINE_RATE   0x01
+
+/* motor types */
+#define MT_OUTPUTSTATE 0x00
+#define MT_ONCHIP_PWM  0x01
+
+/* motor step types */
+#define STT_FULL    0x00	/* 90    degrees */
+#define STT_HALF    0x01	/* 45    degrees */
+#define STT_QUART   0x02	/* 22.5  degrees */
+#define STT_OCT     0x03	/* 11.25 degrees */
+
+/* motor options */
+#define MTR_BACKWARD 0x00
+#define MTR_FORWARD  0x08
+#define MTR_ENABLED  0x00
+#define MTR_DISABLED 0x10
+
+/* sensors */
+#define CCD_SENSOR  0x01
+#define CIS_SENSOR  0x00
+
+/* sony sensor models */
+#define SNYS575     0x00
+
+/* toshiba sensor models */
+#define TCD2952     0x01
+#define TCD2958     0x02
+#define TCD2905     0x03
+
+/* usb types */
 #define USB20       0x01
 #define USB11       0x00
 
+/* scan types */
 #define ST_NEG      0x03
 #define ST_TA       0x02
 #define ST_NORMAL   0x01
 
+/* colour modes */
 #define CM_COLOR    0x00
 #define CM_GRAY     0x01
 #define CM_LINEART  0x02
 
+/* colour channels */
 #define CL_RED      0x00
 #define CL_GREEN    0x01
 #define CL_BLUE     0x02
 
+/* lamp types */
 #define FLB_LAMP    0x01
 #define TMA_LAMP    0x02
 
@@ -72,33 +135,57 @@
 #define TRUE        0x01
 #define FALSE       0x00
 
+/* function results */
 #define OK          0x00
 #define ERROR       -1
 
+#define RT_BUFFER_LEN 0x71a
+
 #define FIX_BY_HARD 0x01
 #define FIX_BY_SOFT 0x02
 
 #define REF_AUTODETECT      0x02
 #define REF_TAKEFROMSCANNER 0x01
+#define REF_NONE            0x00
 
-#define RSZ_GRAYL   0x00
-#define RSZ_COLOURL 0x01
-#define RSZ_COLOURH 0x02
-#define RSZ_LINEART 0x03
-#define RSZ_GRAYH   0x04
+/* bulk operations */
+#define BLK_WRITE    0x00
+#define BLK_READ     0x01
 
+/* constants for resizing functions */
+#define RSZ_NONE     0x00
+#define RSZ_DECREASE 0x01
+#define RSZ_INCREASE 0x02
+
+#define RSZ_GRAYL    0x00
+#define RSZ_COLOURL  0x01
+#define RSZ_COLOURH  0x02
+#define RSZ_LINEART  0x03
+#define RSZ_GRAYH    0x04
+
+/* Macros for managing data */
 #define _B0(x)      ((SANE_Byte)((x) & 0xFF))
 #define _B1(x)      ((SANE_Byte)((x) >> 0x08))
 #define _B2(x)      ((SANE_Byte)((x) >> 0x10))
 #define _B3(x)      ((SANE_Byte)((x) >> 0x18))
 
+/* constants used in RTS_GetImage */
+#define OP_STATIC_HEAD 0x00000001
+#define OP_COMPRESSION 0x00000004
+#define OP_BACKWARD    0x00000010
+#define OP_WHITE_SHAD  0x00000020
+#define OP_USE_GAMMA   0x00000040
+#define OP_BLACK_SHAD  0x00000080
+#define OP_LAMP_ON     0x00000200
+
 /* data types */
 
 typedef unsigned short USHORT;
 
 #ifdef STANDALONE
 
-#define SANE_STATUS_GOOD 0
+#define SANE_STATUS_GOOD 0x00
+
 typedef unsigned char SANE_Byte;
 typedef int SANE_Int;
 
@@ -108,17 +195,35 @@ typedef int SANE_Int;
 
 struct st_debug_opts
 {
+  /* device capabilities */
+  SANE_Int dev_model;
+  SANE_Int chip_model;
+  SANE_Int dev_minres;
+  SANE_Int dev_maxres;
+
   SANE_Byte SaveCalibFile;
   SANE_Byte DumpShadingData;
   SANE_Byte ScanWhiteBoard;
   SANE_Byte EnableGamma;
+  SANE_Byte use_fixed_pwm;
+  SANE_Int dmabuffersize;
+  SANE_Int dmatransfersize;
+  SANE_Int dmasetlength;
+
+  SANE_Int calibrate;
+
+  SANE_Int overdrive_flb;
+  SANE_Int overdrive_ta;
+  SANE_Byte warmup;
 };
 
+
 struct st_scanning
 {
   SANE_Byte *imagebuffer;
   SANE_Byte *imagepointer;
   SANE_Int bfsize;
+  SANE_Int channel_size;
 
   /* Pointers to each channel colour */
   SANE_Byte *pColour[3];
@@ -138,6 +243,9 @@ struct st_resize
   SANE_Int fromwidth;
   SANE_Int towidth;
   SANE_Int bytesperline;
+  SANE_Int rescount;
+  SANE_Int resolution_x;
+  SANE_Int resolution_y;
 
   SANE_Byte *v3624;
   SANE_Byte *v3628;
@@ -168,35 +276,20 @@ struct st_readimage
   SANE_Byte Cancel;
 };
 
-struct st_calib_table
+struct st_gain_offset
 {
-  /* 32 bytes
-     08be|08e0|3654
+  /* 32 bytes 08be|08e0|3654
      red green blue */
-  USHORT edcg1[3];		/* 08e0|08e2|08e4 */
-  USHORT edcg2[3];		/* 08e6|08e8|08ea */
-  USHORT odcg1[3];		/* 08ec|08ee|08f0 */
-  USHORT odcg2[3];		/* 08f2|08f4|08f6 */
+  SANE_Int edcg1[3];		/* 08e0|08e2|08e4 *//*Even offset 1 */
+  SANE_Int edcg2[3];		/* 08e6|08e8|08ea *//*Even offset 2 */
+  SANE_Int odcg1[3];		/* 08ec|08ee|08f0 *//*Odd  offset 1 */
+  SANE_Int odcg2[3];		/* 08f2|08f4|08f6 *//*Odd  offset 2 */
   SANE_Byte pag[3];		/* 08f8|08f9|08fa */
   SANE_Byte vgag1[3];		/* 08fb|08fc|08fd */
   SANE_Byte vgag2[3];		/* 08fe|08ff|0900 */
 };
 
-struct st_icolours
-{
-  SANE_Int red;
-  SANE_Int green;
-  SANE_Int blue;
-};
-
-struct st_dcolours
-{
-  double red;
-  double green;
-  double blue;
-};
-
-struct st_calibration_data
+struct st_calibration_config
 {
   SANE_Int WStripXPos;
   SANE_Int WStripYPos;
@@ -220,8 +313,8 @@ struct st_calibration_data
   SANE_Int OffsetEven2[3];
   SANE_Int OffsetOdd2[3];
   SANE_Byte OffsetHeight;
-  SANE_Byte OffsetPixelStart;
-  SANE_Byte OffsetNPixel;
+  SANE_Int OffsetPixelStart;
+  SANE_Int OffsetNPixel;
   SANE_Byte OffsetNSigma;
   SANE_Int AdcOffPredictStart;
   SANE_Int AdcOffPredictEnd;
@@ -233,6 +326,8 @@ struct st_calibration_data
   SANE_Int CalibGain20n;
   SANE_Int CalibPAGOn;
   SANE_Int GainHeight;
+  SANE_Int unk1[3];
+  SANE_Int unk2[3];
   SANE_Byte PAG[3];
   SANE_Byte Gain1[3];
   SANE_Byte Gain2[3];
@@ -255,13 +350,13 @@ struct st_calibration_data
 struct st_calibration
 {
   /* faac */
-  struct st_calib_table mitabla;	/* 0..35 */
+  struct st_gain_offset gain_offset;	/* 0..35 */
   USHORT *white_shading[3];	/* +36 +40 +44 */
   USHORT *black_shading[3];	/* +48 +52 +56 */
   SANE_Int WRef[3];		/* +60 +62 +64 */
-  SANE_Byte value2;		/* +66 */
-  SANE_Byte value3;		/* +67 */
-  SANE_Int value;		/* +68 */
+  SANE_Byte shading_type;	/* +66 */
+  SANE_Byte shading_enabled;	/* +67 */
+  SANE_Int first_position;	/* +68 */
   SANE_Int shadinglength;	/* +72 */
 };
 
@@ -270,7 +365,7 @@ struct st_cal2
   /* f9f8  35 bytes */
   SANE_Int table_count;		/* +0  f9f8 */
   SANE_Int shadinglength1;	/* +4  f9fc */
-  SANE_Int shadinglength2;	/* +8  fa00 */
+  SANE_Int tables_size;		/* +8  fa00 */
   SANE_Int shadinglength3;	/* +12 fa04 */
   USHORT *tables[4];		/* +16+20+24+28  fa08 fa0c fa10 fa14 */
   USHORT *table2;		/* +32 fa18 */
@@ -339,12 +434,12 @@ struct st_hwdconfig		/* 28 bytes size */
   SANE_Byte compression;	/* +10 */
   SANE_Byte use_gamma_tables;	/* +11 */
   SANE_Byte gamma_tablesize;	/* +12 */
-  SANE_Byte unk1;		/* +13 */
-  SANE_Byte unk2;		/* +14 */
+  SANE_Byte white_shading;	/* +13 */
+  SANE_Byte black_shading;	/* +14 */
   SANE_Byte unk3;		/* +15 */
   SANE_Byte motorplus;		/* +16 */
   SANE_Byte static_head;	/* +17 */
-  SANE_Byte depth;		/* +18 */
+  SANE_Byte motor_direction;	/* +18 */
   SANE_Byte dummy_scan;		/* +19 */
   SANE_Byte highresolution;	/* +20 */
   SANE_Byte sensorevenodddistance;	/* +21 */
@@ -352,16 +447,11 @@ struct st_hwdconfig		/* 28 bytes size */
   SANE_Int calibrate;		/* +24 */
 };
 
-struct st_scandata
+struct st_calibration_data
 {
-  SANE_Int totalcount;
-  SANE_Byte count;
-  SANE_Int *mode;
-};
-
-struct st_scantypes
-{
-  struct st_scandata color[3];
+  SANE_Byte Regs[RT_BUFFER_LEN];
+  struct st_scanparams scancfg;
+  struct st_gain_offset gain_offset;
 };
 
 struct st_cph
@@ -377,56 +467,64 @@ struct st_timing
 {
   SANE_Int sensorresolution;
   SANE_Byte cnpp;
-  SANE_Byte cdss1;
-  SANE_Byte cdsc1;
-  SANE_Byte cdss2;
-  SANE_Byte cdsc2;
-  SANE_Byte cvtrp[3];
+  SANE_Byte cvtrp[3];		/* 3 transfer gates */
   SANE_Byte cvtrw;
-  SANE_Byte cvtrbpw;
   SANE_Byte cvtrfpw;
+  SANE_Byte cvtrbpw;
+  struct st_cph cph[6];		/* Linear Image Sensor Clocks */
+  SANE_Int cphbp2s;
+  SANE_Int cphbp2e;
   SANE_Int clamps;
   SANE_Int clampe;
+  SANE_Byte cdss[2];
+  SANE_Byte cdsc[2];
+  SANE_Byte cdscs[2];		/* Toshiba T958 ccd from hp4370 */
   double adcclkp[2];
   SANE_Int adcclkp2e;
-  struct st_cph cph[6];
 };
 
 struct st_scanmode
 {
+  SANE_Int scantype;
+  SANE_Int colormode;
+  SANE_Int resolution;
+
+  SANE_Byte timing;
+  SANE_Int motorcurve;
   SANE_Byte samplerate;
   SANE_Byte systemclock;
   SANE_Int ctpc;
+  SANE_Int motorbackstep;
+  SANE_Byte scanmotorsteptype;
+
   SANE_Byte dummyline;
   SANE_Int expt[3];
   SANE_Int mexpt[3];
-  SANE_Byte motorplus;
-  SANE_Byte timing;
-  SANE_Int motorcurve;
-  SANE_Int motorbackstep;
-  SANE_Byte multiexposurefor16bitmode;
-  SANE_Byte multiexposureforfullspeed;
-  SANE_Byte has_curves;
+  SANE_Int motorplus;
+  SANE_Int multiexposurefor16bitmode;
+  SANE_Int multiexposureforfullspeed;
+  SANE_Int multiexposure;
   SANE_Int mri;
   SANE_Int msi;
   SANE_Int mmtir;
   SANE_Int mmtirh;
   SANE_Int skiplinecount;
-  SANE_Byte scanmotorsteptype;
 };
 
-struct stmotormove
+struct st_motormove
 {
-  SANE_Byte systemclock;	/* + 0 */
-  SANE_Byte v12e435;		/* + 1 */
-  SANE_Byte scanmotorsteptype;	/* + 2 */
-  USHORT motorcurve;		/* + 4  word */
-  /* + 6..7 */
-  SANE_Int coord_y;		/* + 8 */
-  SANE_Int ctpc;		/* + 12 dword */
-  SANE_Int has_curves;		/* + 16 dword */
-  SANE_Int v12e448;		/* + 20 */
-  SANE_Int v12e44c;		/* + 24 */
+  SANE_Byte systemclock;
+  SANE_Int ctpc;
+  SANE_Byte scanmotorsteptype;
+  SANE_Int motorcurve;
+};
+
+struct st_motorpos
+{
+  SANE_Int coord_y;
+  SANE_Byte options;
+  SANE_Int v12e448;
+  SANE_Int v12e44c;
 };
 
 struct st_find_edge
@@ -461,39 +559,74 @@ struct st_dec
   SANE_Byte scanbufferfull;
 };
 
-struct st_ccurve
+struct st_curve
 {
-  SANE_Int minspeed;		/* word & 0xffff */
-  SANE_Int maxspeed;
-  SANE_Int count;
+  SANE_Int crv_speed;		/* acceleration or deceleration */
+  SANE_Int crv_type;
+  SANE_Int step_count;
   SANE_Int *step;
 };
 
-struct st_motorsetting
+struct st_motorcurve
 {
-  SANE_Int acccurvecount;
-  SANE_Int deccurvecount;
   SANE_Int mri;
   SANE_Int msi;
   SANE_Int skiplinecount;
   SANE_Int motorbackstep;
-  struct st_acc acc;
-  struct st_dec dec;
-  struct st_ccurve *acccurve;	/* acccurves */
-  struct st_ccurve *deccurve;	/* deccurves */
+  SANE_Int curve_count;
+  struct st_curve **curve;
 };
 
 struct st_checkstable
 {
   double diff;
   SANE_Int interval;
-  SANE_Int tottime;
+  long tottime;
 };
 
+struct st_sensorcfg
+{
+  SANE_Int type;
+  SANE_Int name;
+  SANE_Int resolution;
+
+  SANE_Int channel_color[3];
+  SANE_Int channel_gray[2];
+  SANE_Int rgb_order[3];
+
+  SANE_Int line_distance;
+  SANE_Int evenodd_distance;
+};
+
+struct st_autoref
+{
+  SANE_Byte type;
+  SANE_Int offset_x;
+  SANE_Int offset_y;
+  SANE_Int resolution;
+  SANE_Int extern_boundary;
+};
+
+struct st_motorcfg
+{
+  SANE_Byte type;
+  SANE_Int resolution;
+  SANE_Byte pwmfrequency;
+  SANE_Int basespeedpps;
+  SANE_Int basespeedmotormove;
+  SANE_Int highspeedmotormove;
+  SANE_Int parkhomemotormove;
+  SANE_Byte changemotorcurrent;
+};
+
+struct st_buttons
+{
+  SANE_Int count;
+  SANE_Int mask[6];		/* up to 6 buttons */
+};
 
 /* Unknown vars */
-SANE_Int v14b4 = 0x00;
-char v15fc;			/* char */
+SANE_Int v14b4 = 0;
 SANE_Byte *v1600 = NULL;	/* tabla */
 SANE_Byte *v1604 = NULL;	/* tabla */
 SANE_Byte *v1608 = NULL;	/* tabla */
@@ -519,69 +652,29 @@ SANE_Int smearacccurvecount;	/* counter used y MotorSetup */
 SANE_Int smeardeccurvecount;	/* counter used y MotorSetup */
 
 /* Known vars */
-SANE_Int offset_red;		/* USB Address 0070 */
-SANE_Int offset_green;		/* USB Address 0072 */
-SANE_Int offset_blue;		/* USB Address 0074 */
+SANE_Int offset[3];
+SANE_Byte gain[3];
 
-SANE_Byte gain_blue = 0;	/* byte */
-SANE_Byte gain_green = 0;	/* byte */
-SANE_Byte gain_red = 0;		/* byte */
-
-SANE_Byte BGeneral[0x71a];	/* Buffer general */
-static SANE_Int timingfile = -1;
+SANE_Byte BGeneral[RT_BUFFER_LEN];	/* Buffer general */
 static SANE_Int usbfile = -1;
-SANE_Int ccdtype;
 SANE_Int usbtype;
 SANE_Byte vcolormode;
 SANE_Int scantype;
-SANE_Int warmupt1;
-SANE_Int warmupt2;
 
 SANE_Byte pwmlampenabled;
 SANE_Byte pwmlamplevel;
-SANE_Byte pwmlampfrequencyflb1200;
-SANE_Byte pwmlampdutycycleflb1200;
-SANE_Byte pwmlampfrequencyflb600;
-SANE_Byte pwmlampdutycycleflb600;
-SANE_Byte pwmlampfrequencyta1200;
-SANE_Byte pwmlampdutycycleta1200;
-SANE_Byte pwmlampfrequencyta600;
-SANE_Byte pwmlampdutycycleta600;
-SANE_Byte pwmlampfrequencyneg1200;
-SANE_Byte pwmlampdutycycleneg1200;
-SANE_Byte pwmlampfrequencyneg600;
-SANE_Byte pwmlampdutycycleneg600;
 
 SANE_Byte dpi100Lumping;
-SANE_Byte changemotorcurrent;
 SANE_Byte arrangeline;
-SANE_Byte motorcount;
-SANE_Byte timingcount;
-SANE_Int realsensorresolution;
-SANE_Byte sensorlinedistance;
-SANE_Byte motortype;
-SANE_Byte motorpwmfrequency;
-SANE_Int motorresolution;
 SANE_Byte binarythresholdh;
 SANE_Byte binarythresholdl;
-SANE_Int rgbchannelorder[3] = { 0, 0, 0 };
-SANE_Int sensorchannelcolor[3] = { 0, 0, 0 };
-SANE_Int sensorchannelgray[2] = { 0, 0 };
 
-struct st_scantypes scandata[3];
 
-SANE_Int dmabuffersize = 0;
-SANE_Int dmasetlength = 0;
-SANE_Int dmatransfersize = 0;
 SANE_Byte shadingbase;
 SANE_Byte shadingfact[3];
 SANE_Byte arrangeline;
-SANE_Int calibrate;
-SANE_Int throughput;
 SANE_Int compression;
 
-struct st_calib_table default_calib_table;
-
 SANE_Byte linedarlampoff;
 SANE_Byte linedarklevel1;
 SANE_Byte linedarklevel2;
@@ -589,68 +682,29 @@ SANE_Int pixeldarklevel;
 SANE_Int pixeldarkscanlines;
 SANE_Int pixeldarkdiscardpercent;
 SANE_Int waittimeafterpixeldark;
-SANE_Int discardwhitepercent1;
-SANE_Int discardwhitepercent2;
-SANE_Int discardwhitepercent3;
-SANE_Int discardblackpercent1;
-SANE_Int discardblackpercent2;
-SANE_Int discardblackpercent3;
-SANE_Byte sensortype;
-SANE_Byte sensorevenodddistance;
-SANE_Int realsensorresolution;
 SANE_Int lampofftimer;
-SANE_Int lampwarmupchecklevel;
-SANE_Byte binarythresholdh;
-SANE_Byte binarythresholdl;
-SANE_Int basetargetcode;
-char basetarget1;
-char basetarget2;
-SANE_Int basespeedpps;
-SANE_Int basespeedmotormove;
-SANE_Int highspeedmotormove;
-SANE_Int parkhomemotormove;
-SANE_Byte motormovecount = 0x00;
-SANE_Int usefixedpwm;
+
 SANE_Int bw_threshold = 0x00;
 
-struct stmotormove *motormove = NULL;
-struct st_find_edge findedge[3];
-struct st_motorsetting *motorsetting = NULL;
-
-struct st_checkstable check_preview;
-struct st_checkstable check_flb;
-struct st_checkstable check_tma;
-
-struct st_gammatables hp_gamma;
-struct st_scanmode *scanmodeparams = NULL;
-struct st_timing *scantiming = NULL;
-
 /* SetScanParams */
 struct st_scanparams scan;
 struct st_scanparams scan2;
 
-SANE_Int firstconnection = 0;
-SANE_Int findref;
 SANE_Int bytesperline;		/* width * (3 colors [RGB]) */
 SANE_Int imagewidth3;
 SANE_Int lineart_width;
-double dimageheight;		/* double */
 SANE_Int imagesize;		/* bytesperline * coords.height */
-SANE_Int imageresx;
-SANE_Int imageresy;
 SANE_Int imageheight;
-SANE_Int imageheight2;
 SANE_Int use_sensorevenodddist;
 SANE_Int v1594;
-SANE_Int v158c;
+SANE_Int line_size;
 SANE_Int v15b4;
-SANE_Int v15b8;
+SANE_Int arrange_size;
 SANE_Int v15bc;
 SANE_Int v15ac;
 SANE_Int use_compresion;
 SANE_Int v15f4;
 SANE_Int use_hightresolution;
-SANE_Byte scan_samplerate;
 
 SANE_Int v06c0;
 long ltickcount;
@@ -671,7 +725,6 @@ SANE_Int v11d4;
 SANE_Int v11d8;
 SANE_Int v15fa;
 SANE_Int waitforpwm;
-SANE_Int v14b0;
 
 SANE_Byte WRef[3];
 
@@ -679,13 +732,10 @@ USHORT *fixed_black_shading[3] = { NULL, NULL, NULL };
 USHORT *fixed_white_shading[3] = { NULL, NULL, NULL };
 
 /* Calibration */
-struct st_scanparams cpyscan;
-SANE_Byte bkRegisters[0x71a];
-struct st_calib_table mitabla;	/* calibration table */
-struct st_calib_table mitabla2;	/* calibration table */
+struct st_gain_offset mitabla2;	/* calibration table */
 SANE_Int v0750;
 
-static SANE_Byte use_gamma_tables = TRUE;
+static SANE_Byte use_gamma_tables;
 
 SANE_Int read_v15b4 = 0;
 
@@ -696,11 +746,44 @@ SANE_Int v07c0 = 0;
 
 SANE_Int orderchannel = FALSE;
 
-SANE_Int channel_size;
-SANE_Int sresolutiony;
+/* next structure will contain general configuration of stepper motor */
+struct st_motorcfg *motorcfg;
 
-struct st_readimage Reading;
-struct st_constrains *constrains = NULL;
-struct st_debug_opts RTS_Debug;
-struct st_resize Resize;
-struct st_scanning scanning;
+/* next structure will contain general configuration of ccd sensor */
+struct st_sensorcfg *sensorcfg;
+
+/* next structure will contain all ccd timing values */
+SANE_Int timings_count;
+struct st_timing **timings;
+
+/* next structure will contain all possible motor movements */
+SANE_Int motormove_count;
+struct st_motormove **motormove;
+
+/* next extructure will contain all motorcurve values */
+SANE_Int mtrsetting_count;
+struct st_motorcurve **mtrsetting;
+
+/* next structure will contain all possible scanning modes for one scanner */
+SANE_Int scanmodes_count;
+struct st_scanmode **scanmodes;
+
+/* next extructure contains constrain values for one scanner */
+struct st_constrains *constrains;
+
+/* next extructure contains supported buttons and their order */
+struct st_buttons *buttons;
+
+struct st_gammatables *hp_gamma;
+struct st_gain_offset *default_gain_offset;
+struct st_calibration_data *calibdata;
+
+struct st_readimage *Reading;
+struct st_debug_opts *RTS_Debug;
+struct st_resize *Resize;
+struct st_scanning *scanning;
+
+SANE_Byte *jkd_white = NULL;
+SANE_Byte *jkd_black = NULL;
+SANE_Int jkd_whitebpl;
+SANE_Int jkd_blackbpl;
